package studium.simmod;

import android.app.Activity;
import android.media.AudioFormat;
import android.media.AudioManager;
import android.media.AudioRecord;
import android.media.AudioTrack;
import android.media.MediaRecorder;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;


public class AudioClassifierMain extends Activity implements Runnable {   
    
	private static final String ACTIVITY_NAME = AudioClassifierMain.class.getSimpleName();
	
	
	private final int AUDIO_ENCODING = AudioFormat.ENCODING_PCM_16BIT;
	private final int FREQUENCY = 22050; //Herz
	
	private final double REC_DURATION = 5.0; // in seconds
	 
	private static final boolean ACQUISITION_MODE = true;	// true (false) ....... recorded audio data is (not) stored on SD-card
	
	private static final boolean PLAYBACK_MODE = true;	// true (false)...... recorded audio is (not) played back
	
	private static final boolean CLASSIFICATION_MODE = true; // true (false)...... recorded audio is (not) classified
	
	Button processButton = null;
	
	
	private TextView display;
    String displayText;
    final Handler displayHandler = new Handler();  
    
    // Create runnable for updating display
    final Runnable updateDisplayThread = new Runnable() { public void run() { updateDisplay(displayText);}};
    
    int bufferSize = -1;    
 
    public AudioRecord audioRecord = null;

    public AudioTrack audioTrack = null;
    
    short [] buffer = null;
	
	/////////////////////////////////////////////////////////////////////////////////////
	
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);   

        display = (TextView) findViewById(R.id.status);

        processButton = (Button) findViewById(R.id.record);
        processButton.setOnClickListener(new View.OnClickListener() {
            public void onClick(View view) {
                process();
            }
        });

        
        initAudio();
    }
    
    private void initAudio() {
		
        int minBufferSize = AudioTrack.getMinBufferSize(FREQUENCY, AudioFormat.CHANNEL_OUT_MONO, AUDIO_ENCODING);    

       
        if(minBufferSize == AudioRecord.ERROR){
        	
        	Log.e(ACTIVITY_NAME, "init(): Unable to query output properties for audio hardware!");
        	System.exit(-1);
        }
        
        if(minBufferSize == AudioRecord.ERROR_BAD_VALUE){
        	
        	Log.e(ACTIVITY_NAME, "init(): Recording parameters not supported by target device!");
        	System.exit(-1);
        }
        
        bufferSize = (int)(REC_DURATION * 10 * minBufferSize);
        
        
        
        // Create new AudioRecord object to record the audio. 
        audioRecord = new AudioRecord(
        		MediaRecorder.AudioSource.MIC, 
                FREQUENCY, AudioFormat.CHANNEL_IN_MONO, 
                AUDIO_ENCODING, bufferSize);
        
        
        // Create new AudioTrack object w/same parameters as AudioRecord obbj
        audioTrack = new AudioTrack(
                AudioManager.STREAM_MUSIC,
                FREQUENCY, AudioFormat.CHANNEL_OUT_MONO, 
                AUDIO_ENCODING,
                4096, 
                AudioTrack.MODE_STREAM);
        
        
        buffer = new short[bufferSize];
        
        Log.i(ACTIVITY_NAME, "init(): audio buffer created; bufferSize = " + bufferSize);
		
	}

    /////////////////////////////////////////////////////////////////////////////////////////
    
	private void updateDisplay(String update_txt) {
        display.setText(update_txt);
    }

    public void record() {
    	
        try {
            audioRecord.startRecording();
            audioRecord.read(buffer, 0, bufferSize);
            audioRecord.stop();
        } catch (Throwable t) {
            Log.e(ACTIVITY_NAME,"record(): Recording Failed");
        }   
    }
	
	private void playBack(){
		
        audioTrack.play();
        int i=0;
        while(i<bufferSize) {
            audioTrack.write(buffer, i++, 1);
        }
	}

	private void saveToSDCard(){

        java.text.SimpleDateFormat sdf = new java.text.SimpleDateFormat("yyyy.MM.dd HH.mm.ss");
        String fileName = sdf.format(new java.util.Date());
        
        
        Log.i(ACTIVITY_NAME, "saveToSDCard(): start writing as text file");
        
        try {
        	
            String rootS = Environment.getExternalStorageDirectory().toString();
            File root = Environment.getExternalStorageDirectory();
            
            if (root.canWrite()){
            	
                File f = new File(rootS + "/AudioClassifier", fileName + ".txt");
                FileWriter fw = new FileWriter(f);
                BufferedWriter out = new BufferedWriter(fw);
                
                for(int i = 0; i < bufferSize / 2; i++){
                	out.write("" + buffer[i] + " ");
                }
                out.close();
                fw.close();
            }
        } catch (IOException e) {
        	e.printStackTrace();
        }
        Log.i(ACTIVITY_NAME, "saveToSDCard(): finished writing as text file");
        
        
        // TODO: save buffer[] as uncompressed audio file 
		
	}

	private String classify() {
		
		String genre; // ROCK / CLASSIC
		
		//  -----------------------------------------------------------------
		
		// TODO: classify buffer[]against {"ROCK", "CLASSIC"}
		
		// -----------------------------------------------------------------
		
		// Stub implementation ----
		if(buffer[1000] > 0)
			genre = "ROCK";
		else
			genre = "CLASSIC";
		// ------------------------
		
		return genre;
		
	}    
	
    private void process() {
        Thread mainThread = new Thread(new Runnable() {
            public void run() {
            	
                displayText = "recording ...";
                displayHandler.post(updateDisplayThread);
                record();

                if(PLAYBACK_MODE){
                	
                    displayText = "play back ...";
                    displayHandler.post(updateDisplayThread);
                	playBack();
                }
                
                if(ACQUISITION_MODE){
                	
                    displayText = "saving data to SD card ...";
                    displayHandler.post(updateDisplayThread);
                	saveToSDCard();
                }
                
                if(CLASSIFICATION_MODE){
                	
                    displayText = "classifying ...";
                    displayHandler.post(updateDisplayThread);
                    
                    displayText = classify();
                    
                    displayHandler.post(updateDisplayThread);
                    
                    try {
                    	
                    	// Display classification result for 1.5 seconds
						Thread.sleep(1000);
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
                }
                
                displayText = "Ready!";
                displayHandler.post(updateDisplayThread);
            }
 
        });
        
        mainThread.start();
    }
    
	@Override
	public void run() {
		// TODO Auto-generated method stub
		
	}
}